@@ -4,21 +4,32 @@ from watchdog.observers import Observer
 import base64
 import logging
 import os
+import redis
 import requests
 import signal
 import sys
 import time
+import uuid
 
 FRAMES_PATH = os.getenv("FRAMES_PATH", 'frames')
-HOST_LIST_URL = os.getenv("HOST_LIST_URL", None)
-HOST = os.getenv("HOST", "localhost")
-PORT = int(os.getenv("PORT", 9080))
-PUBLISH_URL_TEMPLATE = os.getenv("PUBLISH_URLS_TEMPLATE", 'http://{host}:{port}/pub?id={channel}')
+
+HTTP_HOST_LIST_URL = os.getenv("HTTP_HOST_LIST_URL", None)
+HTTP_HOST = os.getenv("HTTP_HOST", "localhost")
+HTTP_PORT = int(os.getenv("HTTP_PORT", 9080))
+HTTP_PUBLISH_URL_TEMPLATE = os.getenv("HTTP_PUBLISH_URLS_TEMPLATE", 'http://{host}:{port}/pub?id={channel}')
+http_hosts = requests.get(HTTP_HOST_LIST_URL).json() if HTTP_HOST_LIST_URL else [HTTP_HOST]
+
+REDIS_HOST_LIST_URL = os.getenv("REDIS_HOST_LIST_URL", None)
+REDIS_HOST = os.getenv("REDIS_HOST", "")
+REDIS_PORT = int(os.getenv("REDIS_PORT", 6379))
+REDIS_DB = int(os.getenv("REDIS_DB", 0))
+REDIS_TTL = int(os.getenv("REDIS_TTL", 60))
+redis_hosts = requests.get(REDIS_HOST_LIST_URL).json() if REDIS_HOST_LIST_URL else [REDIS_HOST]
+
 BASE64_ENCODE = "BASE64_ENCODE" in os.environ
 LOG_FILE = os.getenv("LOG_FILE", None)
 LOG_LEVEL = getattr(logging, os.getenv("LOG_LEVEL", "debug").upper())
 logger = logging.getLogger("broadcaster")
-hosts = requests.get(HOST_LIST_URL).json() if HOST_LIST_URL else [HOST]
 
 
 class EventHandler(FileSystemEventHandler):
@@ -34,13 +45,24 @@ def post(path):
         data = content.read()
         if BASE64_ENCODE:
             data = base64.b64encode(data)
-        for host in hosts:
-            url = PUBLISH_URL_TEMPLATE.format(channel=channel, host=host, port=PORT)
+        for http_host in [h for h in http_hosts if h]:
+            url = HTTP_PUBLISH_URL_TEMPLATE.format(channel=channel, host=http_host, port=HTTP_PORT)
             r = requests.post(url, data=data)
             if r.status_code == 200:
                 logger.debug('Pushed {} to {}'.format(path, url))
             else:
                 logger.error(r)
+        for redis_host in [h for h in redis_hosts if h]:
+            try:
+                r = redis.StrictRedis(host=redis_host, port=REDIS_PORT, db=REDIS_DB)
+                key = uuid.uuid4()
+                timestamp = os.path.getmtime(path)
+                r.zadd(channel, timestamp, key)
+                r.setex(key, REDIS_TTL, data)
+                r.zremrangebyscore(channel, "-inf", timestamp - REDIS_TTL)
+                logger.debug('Pushed {} to {}. Key={}, timestamp={}'.format(path, redis_host, key, timestamp))
+            except Exception as err:
+                logger.error(err)
     os.remove(path)
 
 
@@ -65,10 +87,12 @@ def delete_all_files(top):
             logger.debug("Removing old file {}".format(path))
             os.remove(path)
 
+
 def signal_handler(signal, frame):
     logger.warning("Interrupt. Shuting down.")
     sys.exit(0)
 
+
 def run():
     setup_logger()
     logger.info('Started')
